/* Charge mtgLoad en version es5 ou module suivant le navigateur
 * ATTENTION, ce fichier doit rester en es5, car il ne passe pas par babel ni webpack
 * (même s'il est minifié par terser et copié par webpack à chaque compil webpack)
 */
/* eslint-env browser */

/**
 * Un "preLoader" qui chargera display en version es5 ou module es6 suivant les capacités du navigateur,
 * puis l'appellera quand il sera chargé
 * Attention, ce preloader ne fonctionne qu'avec callback, ce n'est qu'après le premier appel que window.mtgLoad pourra retourner une promesse
 * @param {HTMLElement} container
 * @param {SvgOptions} svgOptions
 * @param {MtgOptions} mtgOptions
 * @param {mtgLoadCallback} [cb]
 * @returns Promise<MtgApp|MtgAppPlayer>|undefined (undefined si cb fournie)
 */
function mtgLoad (container, svgOptions, mtgOptions, cb) {
  'use strict'
  // c'est un bout de code dans webpack.config.js qui viendra modifier cette ligne
  var timestamp = '1709657834' // généré par C:\Projets_JavaScript\MathGraphJS\mathgraph_js\webpack.config.js
  var publicPath = 'https://www.mathgraph32.org/js/mtgLoad/' // généré par C:\Projets_JavaScript\MathGraphJS\mathgraph_js\webpack.config.js le Tue Mar 05 2024 17:57:13 GMT+0100 (heure normale d’Europe centrale)
  var script = document.createElement('script')
  let src = publicPath + 'mtgLoad.'
  if ('noModule' in script) {
    // le navigateur gère les modules es6, pas sûr qu'il gère les imports dynamiques mais webpack les transforme
    script.type = 'module'
    src += 'module'
  } else {
    script.type = 'application/javascript'
    src += 'es5'
  }
  src += '.min.js?' + timestamp

  // ça c'est dans tous les cas
  script.crossOrigin = 'anonymous'

  // apparemment c'est mieux de faire dans cet ordre pour une meilleure compatibilité avec tous les navigateurs
  document.body.appendChild(script)

  // le cas cb fournie
  if (typeof cb === 'function') {
    if (cb.length !== 2) console.error(Error('La callback passée à mtgLoad est invalide, elle doit prendre deux arguments, l’éventuelle erreur en premier et l’appli instanciée en 2e'))
    script.addEventListener('load', function onLoadRealLoader () {
      // la fct globale mtgLoad est maintenant le vrai loader, mais on vérifie quand même
      if (window.mtgLoad.isPreLoader || typeof window.mtgLoad !== 'function') {
        return cb(Error('Le chargement de mathgraph a échoué'))
      }
      window.mtgLoad(container, svgOptions, mtgOptions, cb)
    })
    script.src = src
    return
  }

  // pas de cb, on retourne une promesse si le navigateur le gère
  if (typeof Promise !== 'undefined') {
    // le navigateur gère les promesses
    return new Promise((resolve, reject) => {
      script.addEventListener('load', function onLoadRealLoader () {
        if (window.mtgLoad.isPreLoader || typeof window.mtgLoad !== 'function') {
          return reject(Error('Le chargement de mathgraph a échoué (' + src + ')'))
        }
        window.mtgLoad(container, svgOptions, mtgOptions).then(resolve).catch(reject)
      })
      script.src = src
    })
  }

  // sinon c'est un très vieux nagigateur qui gère pas les promesses, et on a pas encore chargé le polyfill
  console.error(Error('Ce navigateur n’implémente pas "Promise", il faut passer une callback à mtgLoad'))
  // on charge quand même, mais il pourra pas récupérer l'objet mtgApp
  script.addEventListener('load', function onLoadRealLoader () {
    if (window.mtgLoad.isPreLoader || typeof window.mtgLoad !== 'function') {
      return console.error(Error('Le chargement de mathgraph a échoué (' + src + ')'))
    }
    window.mtgLoad(container, svgOptions, mtgOptions)
  })
  script.src = src
}

window.mtgLoad = mtgLoad
// pour marquer notre préloader et vérifier qu'il a bien été écrasé par le mtgLoad original
window.mtgLoad.isPreLoader = true
